from setuptools import setup

setup(
    name='MCServerInterface',
    version='0.0.2',
    packages=['mcserver'],
    url='https://github.com/TeamNightSky/MCServerWrapper',
    license='GNU License',
    author='Pixymon',
    author_email='nlarsen23.student@gmail.com',
    description='Integratable API for Local Minecraft Servers',
    install_requires=['requests']
)
